<?php
/* Copyright (C) 2016 John Valance - All Rights Reserved.
   You may use, distribute and modify this code under the
   terms of the BSD license. See file LICENSE.txt in the
   top-most parent folder distributed with this source code.
   If missing, please write to: johnv@div1sys.com.
*/
require_once('../03-singleton/DB2Connection.php');
require_once 'Person_DM.php';

class PersonMapper {

    public function __construct( )
    {    }

    /*
     * The mapper method
     */
    public static function map(array $data) {
        $person = new Person_DM;
        $person->setId($data['ID']);
        $person->setFirstName($data['FIRSTNAME']);
        $person->setLastName($data['LASTNAME']);
        $person->setDateOfBirth($data['DATEOFBIRTH']);
        return $person;
    }

    /*
     * FINDER METHODS
     */
    public function find( $id ) {
        $conn = DB2Connection::getInstance();

        $select_row_sql = "select * from person where id = ?";

        $select_row_stmt = db2_prepare($conn, $select_row_sql);
        if (!$select_row_stmt) throw new Exception("Select ROW prepare failed! : ". db2_stmt_errormsg());

        if (!db2_execute($select_row_stmt, array($id)))
            throw new Exception("db2_execute() failed! : ". db2_stmt_errormsg());

        $row = db2_fetch_assoc($select_row_stmt);

        if (!is_array($row)) {
            throw new Exception("Person record not found for id = $id");
        } else {
            return $this->map($row);
        }
    }

    public static function find_all() {
        $conn = DB2Connection::getInstance();

        $sql = "select * from person";

        $stmt = db2_prepare($conn, $sql);
        if (!$stmt) throw new Exception("Select prepare failed! : ". db2_stmt_errormsg());

        if (!db2_execute($stmt))
            throw new Exception("db2_execute() failed! : ". db2_stmt_errormsg());;

        $obj_list = array();
        while ($row = db2_fetch_assoc($stmt )) {
            $obj_list[] = PersonMapper::map($row);
        }

        return $obj_list;
    }

    public static function findByLastName( $lastName ) {
        $conn = DB2Connection::getInstance();

        $sql = "select * from person where upper(lastName) like upper('%?%')";

        $stmt = db2_prepare($conn, $sql);
        if (!$stmt) throw new Exception("Select prepare failed! : ". db2_stmt_errormsg());

        if (!db2_execute($stmt, array($lastName)))
            throw new Exception("db2_execute() failed! : ". db2_stmt_errormsg());;

        $obj_list = array();
        while ($row = db2_fetch_assoc($stmt )) {
            $obj_list[] = $this->map($row);
        }

        return $obj_list;
    }

    /*
     * Database updating methods
     */
    public function save( Person_DM $person ) {
        if (!empty($person->getId())) {
            $this->update( $person );
        } else {
            $this->insert( $person );
        }
    }

    public function insert( Person_DM $person ) {
        $person->validateData();
        $conn = DB2Connection::getInstance();

        $insert_sql =
            "insert into person (firstName, lastName, dateOfBirth) values (?, ?, ?)";
        $bindVals = array(
            $person->getFirstName(),
            $person->getLastName(),
            $person->getDateOfBirth()
        );

        $insert_stmt = db2_prepare($conn, $insert_sql);
        if (!$insert_stmt) throw new Exception("Prepare INSERT failed! : ". db2_stmt_errormsg());

        if (!db2_execute($insert_stmt, $bindVals))
            throw new Exception("db2_execute() failed! : ". db2_stmt_errormsg());
        $person->setId(db2_last_insert_id($conn));
        return $person->getId();
    }

    public function update( Person_DM $person ) {
        $person->validateData();
        $conn = DB2Connection::getInstance();

        $update_sql = "update person set firstName = ?, lastName = ?,
                                         dateOfBirth = ? where id = ?";
        $bindVals = array($person->getFirstName(), $person->getLastName(),
                          $person->getDateOfBirth(), $person->getId());

        $update_stmt = db2_prepare($conn, $update_sql);
        if (!$update_stmt) throw new Exception("Prepare UPDATE failed!");

        if (!db2_execute($update_stmt, $bindVals))
            throw new Exception("db2_execute() failed!");

        return true;
    }

    public function delete( Person_DM $person ) {
        $conn = DB2Connection::getInstance();

        $delete_row_sql = "delete from person where id = ?";

        $delete_row_stmt = db2_prepare($conn, $delete_row_sql);
        if (!$delete_row_stmt) throw new Exception("Delete row prepare failed! : ". db2_stmt_errormsg());

        if (!db2_execute($delete_row_stmt, array($person->getId())))
            throw new Exception("db2_execute() failed! : ". db2_stmt_errormsg());
    }

    public static function delete_all( ) {
        $conn = DB2Connection::getInstance();
        if (!db2_exec($conn, "delete from person"))
            throw new Exception("Delete from person failed! : ". db2_stmt_errormsg());
    }

}
