<?php
/* Copyright (C) 2016 John Valance - All Rights Reserved.
   You may use, distribute and modify this code under the
   terms of the BSD license. See file LICENSE.txt in the
   top-most parent folder distributed with this source code.
   If missing, please write to: johnv@div1sys.com.
*/
require_once('../03-singleton/DB2Connection.php');

class Person {

    private $ID;
    private $FIRSTNAME;
    private $LASTNAME;
    private $DATEOFBIRTH;

    public function __construct( )
    {
    }

    /*
     * Static factory method - create a Person object from DB data
     */
    public static function load(array $data) {
        $person = new Person;
        $person->ID = $data['ID'];
        $person->setFIRSTNAME($data['FIRSTNAME']);
        $person->setLASTNAME($data['LASTNAME']);
        $person->setDATEOFBIRTH($data['DATEOFBIRTH']);
        return $person;
    }

    /*
     * Getters and Setters
     */
    public function getID() { return $this->ID; }
    public function getFIRSTNAME() { return $this->FIRSTNAME; }
    public function getLASTNAME() { return $this->LASTNAME; }
    public function getDATEOFBIRTH() { return $this->DATEOFBIRTH; }

    public function setID( $ID ) {
        if (!empty($this->ID)) {
            throw new Exception("Value of ID field is already set and cannot be changed: ID='{$this->ID}'");
        }
        $this->ID = $ID;
    }
    public function setFIRSTNAME($FIRSTNAME) { $this->FIRSTNAME = trim($FIRSTNAME); }
    public function setLASTNAME($LASTNAME) { $this->LASTNAME = trim($LASTNAME); }
    public function setDATEOFBIRTH($DATEOFBIRTH) {
        $dateVar = strtotime($DATEOFBIRTH);
        $dateYMD = date('Y-m-d', $dateVar);

        if ($dateYMD !== $DATEOFBIRTH) {
            throw new Exception("Date of Birth is not a valid date format: '{$DATEOFBIRTH}'");
        } else {
            $this->DATEOFBIRTH = $DATEOFBIRTH;
        }
    }

    /*
     * DOMAIN LOGIC (i.e. business logic) METHODS
     */

    public function validateData() {
        static $emptyMsg = 'is required for insert or update';

        if ( empty($this->getFIRSTNAME() ) )
            throw new Exception("FIRSTNAME $emptyMsg");

        if ( empty($this->getLASTNAME() ) )
            throw new Exception("LASTNAME $emptyMsg");

        if (  empty($this->getDATEOFBIRTH() ) )
            throw new Exception("DATEOFBIRTH $emptyMsg");

        return true;
    }

    public function getFullName( ) {
        return $this->getFIRSTNAME() . ' ' . $this->getLASTNAME();
    }

    public function getDATEOFBIRTHFormatted( ) {
        return date('D M d, Y', strtotime($this->DATEOFBIRTH));
    }

    public function getAge( ) {
        $datetime1 = new DateTime(date('Ymd'));
        $datetime2 = new DateTime($this->DATEOFBIRTH);
        $interval = $datetime2->diff($datetime1);
        return $interval->format('%r%y years, %m months, %d days');
    }

    public function getAgeText( ) {
        $currDate = date('M d, Y');
        return "{$this->getFullName()} was born on {$this->getDATEOFBIRTHFormatted()}, "
        . "and {$this->getFIRSTNAME()}'s age as of $currDate is {$this->getAge()}";

    }

    public function __toString() {
        return <<<PERSON_STRING
    ID: {$this->getID()}
    First Name: {$this->getFIRSTNAME()}
    Last Name: {$this->getLASTNAME()}
    Date of Birth: {$this->getDATEOFBIRTH()}
PERSON_STRING;
    }

}
